# -*- test-case-name: buildbot.test.test_trial.TestRemoteReporter.testEndSuite -*-

from twisted.trial import unittest
if not hasattr(unittest, "failUnlessEqual"):
    raise unittest.SkipTest("this module requires retrial and a new twisted")

from pprint import pformat

from twisted.trial import unittest, runner, reporter
from twisted.trial.unittest import failUnlessEqual, failIfEqual, failIfIdentical
from twisted.trial.interfaces import ITestMethod, ITestSuite, IJellied, IUnjellied
from twisted.internet import defer, protocol, reactor
from twisted.scripts import trial as trialscript
from twisted.application import strports
from twisted.python import failure, reflect, util, log
from twisted.spread import pb

from buildbot.slave import trial as bstrial
import zope.interface as zi

def _gimmeAFailure():
    f = None
    def _foo():
        1/0
    try:
        _foo()
    except:
        f = failure.Failure()
    return f

class BogusTestCase(unittest.TestCase):
    def test_foo(self):
        pass

def getLogEvents(iface=ITestMethod):
    return [{'iface': iface, 'foobar': 'foobar', 'message': ["this is a log message"], 'time': 1095834711.2008381, 'isError': 0, 'system': '-'}]


INSTANCE_ATTRS = dict(stdout = 'woohoo! stdout',
                      stderr = 'woohoo! stderr',
                      startTime = 1095834374.1481941,
                      endTime = 1095834403.4441471,
                      skip = "this test sucks",
                      todo = "Still needs work",
                      logevents = getLogEvents(),
                      runs = 192,
                      status = reporter.SUCCESS)

EXPECTED_METHOD_JELLY = INSTANCE_ATTRS.copy()

EXPECTED_METHOD_JELLY.update({'logevents': ['2004/09/22 02:31 EDT [-] this is a log message\n'],
                       'errors': [],
                       'failures': [],
                       'fullName': 'buildbot.test.test_trial.BogusTestCase.test_foo',
                       'klass': 'buildbot.test.test_trial.BogusTestCase',
                       'module': 'buildbot.test.test_trial',
                       'name': 'test_foo'})

EXPECTED_SUITE_JELLY = {'tests': [EXPECTED_METHOD_JELLY],
                        'couldNotImport': {},
                        'runners': [],
                        'startTime': None,
                        'endTime': None}


class TweakedTestMethod(runner.TestMethod):
    status = None
    
    def _setupInstanceAttrs(self):
        for attr, v in INSTANCE_ATTRS.iteritems():
            setattr(self, attr, v)


class TestJellyingTrialObjects(unittest.TestCase):
    def setUp(self):
        self.tm = TweakedTestMethod(BogusTestCase.test_foo)
        self.tm._setupInstanceAttrs()

    def testJellyTestMethod(self):
        d = IJellied(self.tm)
        expect = EXPECTED_METHOD_JELLY.copy()
        failUnlessEqual(d, expect, "%s != %s" % (pformat(d), pformat(expect)))


SUCCESS = "success"

class BogusBuildslave(pb.Root):
    startTestArg = endTestArg = startSuiteArg = endSuiteArg = None
    def remote_startTest(self, method):
        self.startTestArg = ITestMethod(method)
        return SUCCESS

    def remote_endTest(self, method):
        self.endTestArg = ITestMethod(method)
        return SUCCESS

    def remote_startSuite(self, expectedTests):
        self.startSuiteArg = expectedTests
        return SUCCESS

    def remote_endSuite(self, suite):
        self.endSuiteArg = ITestSuite(suite)
        return SUCCESS


class _WaitToStop(protocol.Factory):
    def __init__(self):
        self.deferred = defer.Deferred()
        self.portno = 0
    def stopFactory(self):
        self.deferred.callback(self.portno)
    
def _findAPort():
    dfac = _WaitToStop()
    t = reactor.listenTCP(0, dfac)
    portno = t.getHost().port
    dfac.portno = portno
    t.stopListening()
    return dfac


class TestRemoteReporter(unittest.TestCase):
    def setUp(self):
        self.buildslave = BogusBuildslave()
        self.portno, self.strp = None, None

        def _connectReporter(portno):
            print "_connectReporter"
            self.rr = bstrial.RemoteReporter(args=portno)
            return self.rr.setUpReporter()
            
        def _startListen(portno):
            print "startListen"
            reactor.listenTCP(portno, pb.PBServerFactory(self.buildslave))
            return portno
        
        def _cb(portno):
            print "got portno %d" % portno
            self.portno = portno
            return portno

        return _findAPort().deferred.addCallback(_cb
                                   ).addCallback(_startListen
                                   ).addCallback(_connectReporter
                                   ).addErrback(log.err)

    def tearDown(self):
        return self.rr.tearDownReporter() 
        
    def testConnectToSlave(self):
        failIfEqual(self.rr.pbroot, None)

    def testStartTest(self):
        t = TweakedTestMethod(BogusTestCase.test_foo)
        t._setupInstanceAttrs()
        def _cb(result):
            failUnlessEqual(result, SUCCESS)
            for k in EXPECTED_METHOD_JELLY:
                failUnlessEqual(EXPECTED_METHOD_JELLY[k], getattr(self.buildslave.startTestArg, k))
            return result
        return self.rr.startTest(t).addCallback(_cb)
        
    def testEndTest(self):
        t = TweakedTestMethod(BogusTestCase.test_foo)
        t._setupInstanceAttrs()
        def _cb(result):
            failUnlessEqual(result, SUCCESS)
            for k in EXPECTED_METHOD_JELLY:
                failUnlessEqual(EXPECTED_METHOD_JELLY[k], getattr(self.buildslave.endTestArg, k))
            return result
        return self.rr.endTest(t).addCallback(_cb)

    def testStartSuite(self):
        expectedTests = 88
        def _cb(result):
            failUnlessEqual(result, SUCCESS)
            failUnlessEqual(expectedTests, self.buildslave.startSuiteArg)
            return result
        return self.rr.startSuite(expectedTests).addCallback(_cb)

    def testEndSuite(self):
        t = TweakedTestMethod(BogusTestCase.test_foo)
        t._setupInstanceAttrs()
        ts = runner.TestSuite(bstrial.RemoteReporter())
        ts.addMethod(t)
        def _cb(result):
            failUnlessEqual(result, SUCCESS)
            print "self.buildslave.endSuiteArg: %r" % (self.buildslave.endSuiteArg,)
            for k in EXPECTED_SUITE_JELLY:
                failUnlessEqual(EXPECTED_SUITE_JELLY[k], getattr(self.buildslave.endSuiteArg, k))
        return self.rr.endSuite(ts).addCallback(_cb)



